<?php
/*
Plugin Name: Hashcash for Contact Form 7
Text Domain: hashcash-for-contact-form-7
Domain Path: /languages
Description: Captcha-less hashcash spam protection for Contact Form 7 contact forms. Requires Contact Form 7 and WP Hashcash.
Version: 0.1.4.1
Author: Abdussamad Abdurrazzaq
Author URI: http://abdussamad.com
Plugin URI: http://abdussamad.com/archives/581-Captcha-less-Hashcash-spam-protection-for-Contact-Form-7.html
License: GPL2
Tags: hashcash, spam prevention, contact form 7, wp hashcash, anti-spam, antispam
*/

/*  Copyright 2012  Abdussamad Abdurrazzaq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

class wpcf7_wphc {
	
	const VERSION = '0.1.4.1';

	function __construct() {
		add_action( 'plugins_loaded', array( $this, 'load_languages' ) );
		register_activation_hook( __FILE__, array( $this, 'install' ) );
		$this->upgrade();
		$this->save_version();
		add_action( 'admin_notices', array( $this, 'post_install_message' ) );
		add_action( 'plugins_loaded', array( $this, 'hook' ), 999 );
	}
	
	public function load_languages() {
		load_plugin_textdomain( 'hashcash-for-contact-form-7', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' );
	}

	private function upgrade() {
		if( $db_ver = get_option( 'wpcf7_wphc_version' ) ) {
			if( version_compare( $db_ver, '0.1.3' ) == -1 ) {
	  			delete_option( 'wpcf7_wphc_post_install_message' );
			}
		}
	}

	private function save_version() {
	  	update_option( 'wpcf7_wphc_version', self::VERSION );
	}

	function hook() {
		//check that contact form 7 and wp hashcash are loaded
	  	if( ! function_exists( 'wpcf7_add_shortcode' ) || ! function_exists( 'wphc_option' ) ) {
			return;
		}
	  	wpcf7_add_shortcode( 'wphc', array( $this, 'shortcode_handler' ), false );
		add_filter( 'wpcf7_validate_wphc', array( $this, 'validate' ), 10, 2 );
		add_action( 'admin_init', array( $this, 'tag_generator' ), 45 );
	}

	function install() {
		set_transient( 'wpcf7_wphc_post_install_message', 1, 300 );
	}

	function post_install_message() {
	  	if( get_transient( 'wpcf7_wphc_post_install_message' ) ) {
			$error = false;
			if( ! function_exists( 'wpcf7_add_shortcode' ) ) { 
				$error = true;
				?> <div class='error'>
					<p>
						<?php
						printf( esc_html__( 'Hashcash for Contact Form 7 requires the Contact Form 7 plugin to be installed and activated. Please install the %1$sContact Form 7 plugin%2$s.', 'hashcash-for-contact-form-7' ), '<a href="http://wordpress.org/extend/plugins/contact-form-7/">', '</a>' );
						?>	
					</p>
				</div>
			<?php
			}
			if( ! function_exists( 'wphc_option' ) ) { 
				$error = true;
				?> <div class='error'>
					<p>
						<?php
						printf( esc_html__( 'Hashcash for Contact Form 7 requires the WP Hashcash plugin to be installed and activated. Please install the %1$sWP Hashcash plugin%2$s.', 'hashcash-for-contact-form-7' ), '<a href="http://wordpress.org/extend/plugins/wp-hashcash/">', '</a>' );
						?>	
					</p>
				</div>
			<?php
			}

			if( ! $error ) {
				?> <div class='updated'>
					<p>
						<?php
						printf( esc_html__( 'Thank you for installing the Hashcash for Contact Form 7 plugin. Please visit the %1$sContact page%2$s to add the hashcash tag to your contact form.', 'hashcash-for-contact-form-7' ), '<a href="admin.php?page=wpcf7">', '</a>' );
						?>	
					</p>
				</div>
			<?php
			}
		  	delete_transient(  'wpcf7_wphc_post_install_message' );
	  	}
	}
	
	function shortcode_handler( $tag ) {
		global $post;
		static $count = 0;
		
		if( ! is_single() && ! is_page() ) {
		  	$html = '<p>' . esc_html__( 'Hashcash spam protection only works on single posts or pages', 'hashcash-for-contact-form-7' ) . '</p>';
		} else {
			if( is_object( $post ) && $post->comment_status == 'open' ) {
		  		$html = '<p>' . esc_html__( 'Comments must be disabled on this post or page if you want to use WordPress hashcash to protect this contact form from spam.', 'hashcash-for-contact-form-7' ) . '</p>';
			} else {
				if( ! $count ) {
					$count ++;
	  				$html  = '<input type="hidden" id="wphc_value" name="wphc_value" value=""/>';
					$html .= '<noscript>' . esc_html__( 'This contact form requires javascript to work. Please enable javascript in your browser or your message will not be sent.', 'hashcash-for-contact-form-7' ) . '</noscript>';
				} else {
		  			$html = esc_html__( 'Only one [wphc] tag per contact form allowed.', 'hashcash-for-contact-form-7' );
				}
			}
		}
		return $html;
	}

	function validate( $result, $tag ) {
		$options = wphc_option();
		$spam = !in_array( $_POST["wphc_value"], $options['key'] );
		if( $spam ) {
		  	$result['valid'] = false;
		}
		return $result;
	}

	function tag_generator() {
		wpcf7_add_tag_generator( 'wphc', esc_html__( 'Hashcash', 'hashcash-for-contact-form-7' ), 'wpcf7-tg-pane-wphc', array( $this, 'tg_pane' ), array( 'nameless' => 1 ) );
	}

	function tg_pane() {
		?>
		<div id="wpcf7-tg-pane-wphc" class="hidden">
			<form action="">
				<div class="tg-tag">
					<?php echo esc_html( esc_html__( "Copy this code and paste it into the form left.", 'wpcf7' ) ); ?>
					<input type="text" name="wphc" class="tag" readonly="readonly" onfocus="this.select()" />
				</div>
			</form>
		</div>  
	<?php
	}
}

new wpcf7_wphc();
